Type: Package
Package: tidynorm
Title: Tools for Tidy Vowel Normalization
Version: 0.4.1
Authors@R: c(
    person("Josef", "Fruehwald", , "JoFrhwld@gmail.com", role = c("cre", "aut", "cph"))
  )
Description: An implementation of tidy speaker vowel normalization.
    This includes generic functions for defining new normalization methods for
    points, formant tracks, and Discrete Cosine Transform coefficients, as well
    as convenience functions implementing established normalization methods.
    References for the implemented methods are:
    Johnson, Keith (2020) <doi:10.5334/labphon.196>
    Lobanov, Boris (1971) <doi:10.1121/1.1912396>
    Nearey, Terrance M. (1978) <https://sites.ualberta.ca/~tnearey/Nearey1978_compressed.pdf>
    Syrdal, Ann K., and Gopal, H. S. (1986) <doi:10.1121/1.393381>
    Watt, Dominic, and Fabricius, Anne (2002) <https://www.latl.leeds.ac.uk/article/evaluation-of-a-technique-for-improving-the-mapping-of-multiple-speakers-vowel-spaces-in-the-f1-f2-plane/>.
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
Imports: checkmate, cli, dplyr, glue, options, purrr, Rcpp, rlang,
        stringr, tidyr, tidyselect
URL: https://jofrhwld.github.io/tidynorm/,
        https://github.com/JoFrhwld/tidynorm
Depends: R (>= 4.1)
Suggests: ggdensity, ggplot2, knitr, magick, quarto, reticulate,
        rmarkdown, testthat (>= 3.0.0), tibble
Config/testthat/edition: 3
Config/testthat/parallel: true
VignetteBuilder: quarto
LinkingTo: Rcpp, RcppArmadillo
BugReports: https://github.com/JoFrhwld/tidynorm/issues
NeedsCompilation: yes
Packaged: 2026-05-10 15:10:56 UTC; joseffruehwald
Author: Josef Fruehwald [cre, aut, cph]
Maintainer: Josef Fruehwald <JoFrhwld@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-10 15:30:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-10 17:12:37 UTC; unix
Archs: tidynorm.so.dSYM
