# Generated by roxygen2: do not edit by hand

S3method(dct,matrix)
S3method(dct,numeric)
S3method(idct,matrix)
S3method(idct,numeric)
export(bark_to_hz)
export(check_norm)
export(dct)
export(dct_basis)
export(hz_to_bark)
export(hz_to_mel)
export(idct)
export(idct_accel)
export(idct_rate)
export(mel_to_hz)
export(norm_barkz)
export(norm_dct_barkz)
export(norm_dct_deltaF)
export(norm_dct_generic)
export(norm_dct_lobanov)
export(norm_dct_nearey)
export(norm_dct_wattfab)
export(norm_deltaF)
export(norm_generic)
export(norm_lobanov)
export(norm_nearey)
export(norm_track_barkz)
export(norm_track_deltaF)
export(norm_track_generic)
export(norm_track_lobanov)
export(norm_track_nearey)
export(norm_track_wattfab)
export(norm_wattfab)
export(reframe_with_dct)
export(reframe_with_dct_smooth)
export(reframe_with_idct)
export(tidynorm_options)
import(cli)
import(dplyr)
import(rlang)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(checkmate,check_character)
importFrom(options,opt)
importFrom(rlang,`!!`)
importFrom(stats,sd)
useDynLib(tidynorm, .registration = TRUE)
