#' @import rlang
#' @import cli
#' @importFrom stats sd
#' @import dplyr
#' @keywords internal
#' ## usethis namespace: start
#' @importFrom Rcpp sourceCpp
#' @importFrom Rcpp evalCpp
#' @useDynLib tidynorm, .registration = TRUE
## usethis namespace: end
"_PACKAGE"
