# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

cos_fun <- function(j, k, N) {
    .Call(`_tidynorm_cos_fun`, j, k, N)
}

cos_bank <- function(jj, kk) {
    .Call(`_tidynorm_cos_bank`, jj, kk)
}

dct_fun <- function(y, kk) {
    .Call(`_tidynorm_dct_fun`, y, kk)
}

dct_mat <- function(y, kk) {
    .Call(`_tidynorm_dct_mat`, y, kk)
}

idct_fun <- function(y, n) {
    .Call(`_tidynorm_idct_fun`, y, n)
}

idct_mat <- function(y, n) {
    .Call(`_tidynorm_idct_mat`, y, n)
}

idct_prime <- function(y, n) {
    .Call(`_tidynorm_idct_prime`, y, n)
}

idct_dprime <- function(y, n) {
    .Call(`_tidynorm_idct_dprime`, y, n)
}

idct_interp <- function(y, n, p) {
    .Call(`_tidynorm_idct_interp`, y, n, p)
}

